package com.unis.common.utils;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.annotations.ApiModelProperty;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.Assert;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;

@Slf4j
public class FieldUtil {

    public static void getPoweredOrderedFields(LinkedHashMap<String,String> headMap, Object... objArr){
        List<Field> list=new ArrayList<>();
        for(Object obj : objArr){
            list.addAll(getPoweredOrderedFields(obj,"asc"));
        }
        ApiModelProperty modelProperty=null;
        String notes=null;
        for(Field field : list){
            String name=field.getName();
            modelProperty= field.getAnnotation(ApiModelProperty.class);
            if(modelProperty!=null){
                if(StringUtils.isNotEmpty(notes=modelProperty.notes())){
                    headMap.put(name, notes);
                }else if(StringUtils.isNotEmpty(notes=modelProperty.value())){
                    headMap.put(name, notes);
                }
            }else{
                headMap.put(name, name.toUpperCase());
            }
        }
    }

    public static List<Field> getPoweredOrderedFields(Object obj, String desc){
        Field[] fieldArr= obj.getClass().getDeclaredFields();
        List<Field> fieldList=new ArrayList<>();
        for(Field field : fieldArr){
            if(field.getAnnotation(Column.class)!=null
                    && (field.getAnnotation(JsonIgnore.class)==null || field.getAnnotation(JsonProperty.class)==null)){
                fieldList.add(field);
            }
        }
        sort(fieldList,"getName",desc);
        return fieldList;
    }

    public static <E>List<E> sort(List<E> list, final String method, final String sort){
        Collections.sort(list, new Comparator<E>(){
            Class<Object>[] clsArr=null;
            public int compare(E a, E b){
                int ret = 0;
                try{
                    Method m1 = a.getClass().getMethod(method, clsArr);
                    Method m2 = b.getClass().getMethod(method, clsArr);
                    if(sort != null && "desc".equals(sort))//倒序
                        ret = String.valueOf(m2.invoke(b)).compareTo(String.valueOf(m1.invoke(a)));
                    else//正序
                        ret = String.valueOf(m1.invoke(a)).compareTo(String.valueOf(m2.invoke(b)));
                }catch(NoSuchMethodException ne){
                    System.out.println(ne);
                }catch(IllegalAccessException ie){
                    System.out.println(ie);
                }catch(InvocationTargetException it){
                    System.out.println(it);
                }
                return ret;
            }
        });
        return list;
    }

    public static void resloveImportFields(String fields, LinkedHashMap<String, String> headMap){
        String[] strArr=fields.split(",");
        String[] tempArr=null;
        for(String field : strArr){
            field=field.trim();
            if(field.contains("-")){
                tempArr= field.split("-", 2);
                headMap.put(tempArr[0], tempArr[1]);
            }else{
                headMap.put(field, field.toUpperCase());
            }
        }
    }

    public static boolean rebuildHeadMap(LinkedHashMap<String, Object> realHeadMap, LinkedHashMap<String, String> entityHeadMap, Set<String> headSet){

        headSet.clear();

        Iterator<Map.Entry<String, String>> it= entityHeadMap.entrySet().iterator();
        Map<String, String> reverseHeadMap= new HashMap<>();
        Map.Entry<String, String> entry=null;
        while(it.hasNext()){
            entry= it.next();
            reverseHeadMap.put(entry.getValue(), entry.getKey());
        }

        LinkedHashMap<String, Object> returnHeadMap= new LinkedHashMap<>();
        Map.Entry<String, Object> entrys=null;
        Iterator<Map.Entry<String, Object>> its= realHeadMap.entrySet().iterator();
        String key=null ;String value=null;
        while(its.hasNext()){
            entrys= its.next();
            key= entrys.getKey();
            value= (String)entrys.getValue();
            if(key!=null){
                key= key.trim();
            }
            if(value!=null){
                value= value.trim();
            }
            if(entityHeadMap.containsKey(key)){
                returnHeadMap.put(key, entityHeadMap.get(key));
            }else if(StringUtils.isNotEmpty(value) && entityHeadMap.containsKey(value)){
                returnHeadMap.put(value, entityHeadMap.get(value));
            } else if(reverseHeadMap.containsKey(key)){
                returnHeadMap.put(reverseHeadMap.get(key), key);
            } else{
                headSet.add(key);
            }
        }
        realHeadMap.clear();
        realHeadMap.putAll(returnHeadMap);
        int size=headSet.size();
        return size==0 ? true : false;
    }

    public synchronized static <S, T> T copyProperties(S source, Class<T> clazz) {
        Assert.notNull(clazz, "clazz must not be null");
        if (source == null) {
            log.debug("copyProperties-source-null->" + clazz);
            return null;
        }
        try {
            T t = clazz.newInstance();
            BeanUtils.copyProperties(source, t);
            return t;
        } catch (Exception e) {
            log.info("CopyPropertiesException -> " + e);
            return null;
        }
    }
}
