package com.unis.common.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.web.multipart.MultipartFile;

import com.unis.common.vo.poi.Column;
import com.unis.common.vo.poi.Line;


public class POI4Util {
	/**
	 * 导出工具类
	 * 
	 * @param response
	 * @param lines    跟导入进来,返回出来的对象一致,行集合
	 * @param titles   为Excel 实体类属性集合一一对应的意思 比如我的实体类有 id,name,age,address
	 *                 那么这个集合就是List<String> list =new
	 *                 ArrayList<>(){"id","名称","年龄","地址"}
	 * @param title    为标题
	 * @throws Exception
	 */
	public static void excelExport(HttpServletResponse response, List<Line> lines, List<String> titles, String title)
			throws Exception {
		response.setHeader("content-Type", "application/vnd.ms-excel");
		// 下载文件的默认名称
		response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(title, "utf-8"));

		exportExcel(titles, lines, response, title);
	}
	
	 /**
	  * @Author lidiaoquan
	  * @version V1.0
	  * @Description post 导出文件 返回文件流   参数过长使用
	  * @Date 2019年11月21日 下午3:14:29
	 * @param response
	 * @param lines    跟导入进来,返回出来的对象一致,行集合
	 * @param titles   为Excel 实体类属性集合一一对应的意思 比如我的实体类有 id,name,age,address
	 *                 那么这个集合就是List<String> list =new
	 *                 ArrayList<>(){"id","名称","年龄","地址"}
	 * @param title    为标题
	 * @throws Exception
	  */
    public static void postDxportExcelFile(HttpServletResponse response, List<Line> lines, List<String> titles, String title){
        try {
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            exportExcelToOs(titles, lines, os, title);
            byte[] content = os.toByteArray();
            InputStream is = new ByteArrayInputStream(content);
            // 设置response参数，可以打开下载页面
            response.reset();

            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8"); 
    		response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(title, "utf-8"));
            response.setContentLength(content.length);
            ServletOutputStream outputStream = response.getOutputStream();
            BufferedInputStream bis = new BufferedInputStream(is);
            BufferedOutputStream bos = new BufferedOutputStream(outputStream);
            byte[] buff = new byte[8192];
            int bytesRead;
            while (-1 != (bytesRead = bis.read(buff, 0, buff.length))) {
                bos.write(buff, 0, bytesRead);

            }
            bis.close();
            bos.close();
            outputStream.flush();
            outputStream.close();
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

	public static void exportExcel(List<String> titles, List<Line> lines, HttpServletResponse response, String title) {
		// 创建Excel工作簿
		HSSFWorkbook workbook = new HSSFWorkbook();
		// 创建一个工作表sheet 默认是表名是sheet0
		HSSFSheet sheet = workbook.createSheet();
		// 创建表的第一行
//		HSSFRow row = sheet.createRow(0);
		sheet.setDefaultColumnWidth(20);
		sheet.setDefaultRowHeightInPoints(20);
		// 创建第0行 也就是标题
//		row.setHeightInPoints(30);// 设备标题的高度
//		sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, titles.size()));

		// 创建单元格
		HSSFCell cell = null;
//		cell = row.createCell(0);
//		cell.setCellStyle(titleStyle(workbook));
//		cell.setCellValue(title);

		HSSFRow row1 = sheet.createRow(0);
//		HSSFRow row1 = sheet.createRow(1);
		// 循环为第一行插入标题
		for (int i = 0; i < titles.size(); i++) {
			cell = row1.createCell(i);
			cell.setCellStyle(style(workbook));
			cell.setCellValue(titles.get(i));
		}
		// 追加数据 1是第二行
		if(lines != null && lines.size() > 0) {
			for (int i = 1; i <= lines.size(); i++) {
				Line line = lines.get(i - 1);
				List<Column> columns = line.getColumns();
				HSSFRow nextrow = sheet.createRow(i);
				for (int j = 0; j < columns.size(); j++) {
					HSSFCell cell2 = nextrow.createCell(j);
					cell2.setCellStyle(style1(workbook));
					if (columns.get(j).getValue() == null) {
						cell2.setCellValue("");
					} else {
						cell2.setCellValue(String.valueOf(columns.get(j).getValue()));
					}
				}
			}
		}
		ServletOutputStream out;
		try {
			// .xls 是2003版本，excel2003、2007、2010都可以打开，兼容性最好
//			response.setContentType("APPLICATION/OCTET-STREAM");
//			response.setHeader("Content-Disposition", "attachment;filename=" + title + ".xls");
			
            response.reset();// 清空输出流
            response.setContentType("application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename="+new String(title.getBytes(), "iso8859-1")+".xls"); 
            response.setCharacterEncoding("UTF-8");
            
			out = response.getOutputStream();
			workbook.write(out);
			workbook.close();
//			out.flush();
//			out.close();			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	public static void exportExcelToOs(List<String> titles, List<Line> lines, OutputStream os, String title) {
		// 创建Excel工作簿
		HSSFWorkbook workbook = new HSSFWorkbook();
		// 创建一个工作表sheet 默认是表名是sheet0
		HSSFSheet sheet = workbook.createSheet();
		// 创建表的第一行
//		HSSFRow row = sheet.createRow(0);
		sheet.setDefaultColumnWidth(20);
		sheet.setDefaultRowHeightInPoints(20);
		// 创建第0行 也就是标题
//		row.setHeightInPoints(30);// 设备标题的高度
//		sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, titles.size()));
		
		// 创建单元格
		HSSFCell cell = null;
//		cell = row.createCell(0);
//		cell.setCellStyle(titleStyle(workbook));
//		cell.setCellValue(title);
		
		HSSFRow row1 = sheet.createRow(0);
//		HSSFRow row1 = sheet.createRow(1);
		// 循环为第一行插入标题
		for (int i = 0; i < titles.size(); i++) {
			cell = row1.createCell(i);
			cell.setCellStyle(style(workbook));
			cell.setCellValue(titles.get(i));
		}
		// 追加数据 1是第二行
		if(lines != null && lines.size() > 0) {
			for (int i = 1; i <= lines.size(); i++) {
				Line line = lines.get(i - 1);
				List<Column> columns = line.getColumns();
				HSSFRow nextrow = sheet.createRow(i);
				for (int j = 0; j < columns.size(); j++) {
					HSSFCell cell2 = nextrow.createCell(j);
					cell2.setCellStyle(style1(workbook));
					if (columns.get(j).getValue() == null) {
						cell2.setCellValue("");
					} else {
						cell2.setCellValue(String.valueOf(columns.get(j).getValue()));
					}
				}
			}
		}
		try {
			workbook.write(os);
			workbook.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	// 标题第一行样式
	public static HSSFCellStyle titleStyle(HSSFWorkbook wb) {
		HSSFCellStyle style = wb.createCellStyle();
		style.setAlignment(HorizontalAlignment.CENTER);
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		style.setBottomBorderColor((short) 8);
		style.setBorderBottom(BorderStyle.THIN); // HSSFCellStyle.BORDER_THIN
		style.setBorderLeft(BorderStyle.THIN);
		style.setBorderRight(BorderStyle.THIN);
		style.setBorderTop(BorderStyle.THIN);
		HSSFFont headerFont = (HSSFFont) wb.createFont(); // 创建字体样式
		headerFont.setFontName("黑体"); // 设置字体类型
		headerFont.setFontHeightInPoints((short) 15); // 设置字体大小
		style.setFont(headerFont); // 为标题样式设置字体样式
		return style;
	}

	// 第二行样式
	public static HSSFCellStyle style(HSSFWorkbook wb) {
		HSSFCellStyle style = wb.createCellStyle();
		style.setAlignment(HorizontalAlignment.CENTER);
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		style.setBottomBorderColor((short) 8);
		style.setBorderBottom(BorderStyle.THIN); // HSSFCellStyle.BORDER_THIN
		style.setBorderLeft(BorderStyle.THIN);
		style.setBorderRight(BorderStyle.THIN);
		style.setBorderTop(BorderStyle.THIN);
		HSSFFont headerFont = (HSSFFont) wb.createFont(); // 创建字体样式
		headerFont.setFontName("黑体"); // 设置字体类型
		headerFont.setFontHeightInPoints((short) 12); // 设置字体大小
		style.setFont(headerFont); // 为标题样式设置字体样式
		return style;
	}

	// 数据行样式
	public static HSSFCellStyle style1(HSSFWorkbook wb) {
		HSSFCellStyle style1 = wb.createCellStyle();
		style1.setWrapText(true);// 设置自动换行
		style1.setAlignment(HorizontalAlignment.CENTER); // HSSFCellStyle.ALIGN_CENTER
		style1.setVerticalAlignment(VerticalAlignment.CENTER); // HSSFCellStyle.VERTICAL_CENTER // 创建一个居中格式
		style1.setBottomBorderColor((short) 8);
		style1.setBorderBottom(BorderStyle.THIN); // HSSFCellStyle.BORDER_THIN
		style1.setBorderLeft(BorderStyle.THIN);
		style1.setBorderRight(BorderStyle.THIN);
		style1.setBorderTop(BorderStyle.THIN);
		HSSFFont headerFont1 = (HSSFFont) wb.createFont(); // 创建字体样式
		headerFont1.setFontName("黑体"); // 设置字体类型
		headerFont1.setFontHeightInPoints((short) 10); // 设置字体大小
		style1.setFont(headerFont1); // 为标题样式设置字体样式
		return style1;
	}
	
	 /**
     * 导入工具类
     *  file Excel :上传过来的文件
     * columnNames :为Excel 实体类属性集合一一对应
     * 比如我的实体类有 id,name,age,address
     * 那么这个集合就是List<String> list =new ArrayList<>(){id,name,age,address}
      */
    public static Map<String, Object> uploadExcel(MultipartFile file, List<String> columnNames) {
        List<Line> lines = new LinkedList<>();
        Map<String, Object> resultMap = new HashMap<String, Object>();
        int state = 1;
        String msg = "读取成功！";
        if (file.isEmpty()) {
            state = 0;
            msg = "文件为空！";
        }
        try {
            //根据路径获取这个操作excel的实例
            Workbook wb = WorkbookFactory.create(file.getInputStream());
            //根据页面index 获取sheet页
            Sheet sheet = wb.getSheetAt(0);
            Row row = null;
            //循环sheet页中数据从第二行开始，第一行是标题
            for (int i = 1; i < sheet.getPhysicalNumberOfRows(); i++) {
               Line line = new Line();
                List<Column> columns = new ArrayList<Column>();
                row = sheet.getRow(i);
                for (int j = 0; j < columnNames.size(); j++) {
                    Column clbo = new Column();
                    clbo.setName(columnNames.get(j));
                    if (row.getCell(j) != null) {
                        CellType cellType = row.getCell(j).getCellType();
                        String value = null;
                        if (CellType.NUMERIC.equals(cellType)) {
                            if ("General".equals(row.getCell(j).getCellStyle().getDataFormatString())) {
                                value = String.valueOf(row.getCell(j).getNumericCellValue());
                            } else if ("m/d/yy".equals(row.getCell(j).getCellStyle().getDataFormatString())) {
                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                value = sdf.format(row.getCell(j).getDateCellValue());
                            } else {
                                value = String.valueOf(row.getCell(j).getNumericCellValue());
                            }
                            clbo.setValue(value);
                        } else {
                            row.getCell(j).setCellType(CellType.STRING);//设置成String
                            clbo.setValue(row.getCell(j).getStringCellValue());
                        }
                    } else {
                        clbo.setValue("");
                    }
                    columns.add(clbo);
                }
                line.setColumns(columns);
                lines.add(line);
            }
        } catch (Exception e) {
            //log.error(e.getMessage());
            e.printStackTrace();
            state = 0;
            msg = "读取失败，请稍后再试！";
        }
        resultMap.put("state", state);
        resultMap.put("msg", msg);
        resultMap.put("lines", lines);
        return resultMap;
    }
    

}
