package com.unis.common.utils;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.NumberToTextConverter;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Map.Entry;


public class PoiModelExcelUtil {
	public final static String DATE_OUTPUT_PATTERNS = "yyyy-MM-dd HH:mm:ss";


	public static Object getCellValue(Cell cell) {
		Object ret;
        switch (cell.getCellType()) {
	        case BLANK:
	            ret = null;
	            break;
	        case BOOLEAN:
	            ret = String.valueOf(cell.getBooleanCellValue());
	            break;
	        case ERROR:
	            ret = null;
	            break;
	        case FORMULA:
	            Workbook wb = cell.getSheet().getWorkbook();
	            CreationHelper crateHelper = wb.getCreationHelper();
	            FormulaEvaluator evaluator = crateHelper.createFormulaEvaluator();
	            ret = getCellValue(evaluator.evaluateInCell(cell));
	            break;
	        case NUMERIC:
	            if (DateUtil.isCellDateFormatted(cell)) {
	            	ret = cell.getDateCellValue();
	            } else {
	                ret = NumberToTextConverter.toText(cell.getNumericCellValue());
	            }
	            break;
	        case STRING:
	            ret = cell.getRichStringCellValue().getString();
	            break;
	        default:
	            ret = null;
	            break;
	        }
        return ret; //有必要自行trim
    }

	public static LinkedHashMap<String, Object> getRowMap(int cells, Row row){
		LinkedHashMap<String, Object> term = new LinkedHashMap<>();
		Cell cell=null;
		int num=0;
		Object obj;
		String comment;
		for (short c = 0; c < cells; c++){
			cell=row.getCell(c);
			if (cell != null){
				obj=getCellValue(cell);
				comment=getCellComment(cell.getCellComment());
				if(StringUtils.isNotEmpty(comment)){
					term.put(comment, obj);
				}else{
					if(StringUtils.isNotEmpty((String)obj)){
						term.put(String.valueOf(obj), obj);
					}else{
						throw new IllegalArgumentException("无法读取表头属性");
					}
				}
				if(obj==null){
					num++;
				}
			}else{
				term.put(String.valueOf(num),null);
				num++;
				throw new IllegalArgumentException("无法读取单元格");
			}
			cell=null;
		}
		if(num==cells){
			return null;
		}else{
			return term;
		}
	}

	public static Map<String,Object> getRowValue(String[] titleArr, int cells, Row row){
		Map<String,Object> term = new HashMap<>();
		Cell cell=null;
		int num=0;
		Object obj;
		String key=null;
		for (short c = 0; c < cells; c++){
			cell=row.getCell(c);
			key=titleArr[c];
			if (cell != null){
				obj=getCellValue(cell);
				term.put(key,obj);
				if(obj==null){
					num++;
				}
			}else{
				term.put(key,null);
				num++;
			}
			cell=null;
		}
		if(num==cells){
			return null;
		}else{
			return term;
		}
	}

	/**
	 * 读取excel单元格注释
	 * @author lipenghe
	 */
	public static String getCellComment(Comment comment){
		if(comment!=null){
			return comment.getString().getString();
		}
		return null;
	}

	public static ResponseWrapper resloveExcel(List<MultipartFile> fileList, LinkedHashMap<String, String> entityHeadMap, List<Map<String, Object>> allSheetData) throws IOException {
		Workbook workbook=null;
		LinkedHashMap<String, Object> headMap=null;
		Set<String> headSet=new HashSet();
		List<Map<String, Object>> sheetData=null;
		for(MultipartFile partFile : fileList){
			if(!partFile.isEmpty()){
				String ext= FilenameUtils.getExtension(partFile.getOriginalFilename());
				workbook= PoiModelExcelUtil.readExcelWorkbook(partFile.getInputStream(), ext);
				if(workbook!=null){
					int sheetNum=workbook.getNumberOfSheets();
					for(int i=0;i<sheetNum;i++){
						headMap= PoiModelExcelUtil.readExcelSheetData(workbook, i, 0, 0);
						if(headMap!=null && headMap.size()>0){
							if(!FieldUtil.rebuildHeadMap(headMap, entityHeadMap, headSet)) {
								return ResponseWrapper.fail(partFile.getOriginalFilename() + "的第" + i + "个sheet页,以下属性不能识别: " + headSet);
							}
						}else{
							return ResponseWrapper.fail(partFile.getOriginalFilename()+"的第"+i+"个sheet页,未读取到表头属性");
						}
					}
					//数据解析
					for(int i=0;i<sheetNum;i++){
						headMap= PoiModelExcelUtil.readExcelSheetData(workbook, i, 0, 0);
						FieldUtil.rebuildHeadMap(headMap, entityHeadMap, headSet);
						if(headMap!=null && headMap.size()>0){
							sheetData=PoiModelExcelUtil.readExcelSheetData(workbook, headMap, i, 1, 0, 0);
							allSheetData.addAll(sheetData);
						}
					}
				}else{
					return ResponseWrapper.fail("解析"+partFile.getOriginalFilename()+"失败");
				}
			}
		}
		return ResponseWrapper.success();
	}
	
	public static Workbook readExcelWorkbook(InputStream inputStream, String ext){
		//工作区
		Workbook wb = null;
		try {
			if("xlsx".equals(ext)){
				wb = new XSSFWorkbook(inputStream);
			}else{
				wb = new HSSFWorkbook(inputStream);
			}
			inputStream.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return wb;
	}

	public static LinkedHashMap<String, Object> readExcelSheetData(Workbook wb, int sheetIndex, int rowNum, int columnNum){
		sheetIndex=sheetIndex<=0?0:sheetIndex;
		int sheetCount = wb.getNumberOfSheets();
		if(sheetCount<sheetIndex){
			return null;
		}else{
			Sheet sheet = wb.getSheetAt(sheetIndex);
			columnNum=columnNum<=0?0:columnNum;
			Row row=null;
			int cells=0;
			LinkedHashMap<String, Object> rowData=null;
			row = sheet.getRow(rowNum);
			if (row != null){
				cells = row.getLastCellNum();
				cells=columnNum>0?columnNum:cells;
				rowData=getRowMap(cells,row);
				row=null;
			}
			return rowData;
		}
	}

	public static List<Map<String, Object>> readExcelSheetData(Workbook wb, LinkedHashMap<String, Object> headMap, int sheetIndex, int rowStart, int rowEnd, int columnNum){
		String[] headArr=null;
		if(headMap!=null){
			headArr=headMap.keySet().toArray(new String[]{});
		}
		return readExcelSheetData(wb, headArr, sheetIndex, rowStart, rowEnd, columnNum);
	}

	public static List<Map<String,Object>> readExcelSheetData(Workbook wb, String[] titleArr, int sheetIndex, int rowStart, int rowEnd, int columnNum){
		sheetIndex=sheetIndex<=0?0:sheetIndex;
		int sheetCount = wb.getNumberOfSheets();
		if(sheetCount<sheetIndex||titleArr==null){
			return null;
		}else{
			List<Map<String,Object>> sheetList=new ArrayList<>();
			Sheet sheet = wb.getSheetAt(sheetIndex);
			int rows =sheet.getLastRowNum()+1;
			rowStart=rowStart<=0?0:rowStart;
			rowEnd=rowEnd<=0?rows:rowEnd;
			columnNum=columnNum<=0?0:columnNum;
			Row row=null;
			int cells=0;
			Map<String,Object> rowData=null;
			for (int r = rowStart; r <rowEnd; r++){
				row = sheet.getRow(r);
				if (row != null){
					cells = row.getLastCellNum();
					cells=columnNum>0?columnNum:cells;
					cells=Math.min(cells, titleArr.length);
					rowData=getRowValue(titleArr,cells,row);
					if(rowData!=null){
						sheetList.add(rowData);
					}
					row=null;
				}
			}
			return sheetList;
		}
	}
}
