package com.unis.common.vo.base;

import com.github.pagehelper.Page;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.util.List;

@Data
@ApiModel(value = "PageResult<T>（分页结果集）", description = "分页结果集")
public class PageResult<T> {

	@ApiModelProperty(value = "总条数")
	private Long total;// 总条数

	@ApiModelProperty(value = "总页数")
	private Integer totalPage;// 总页数

	@ApiModelProperty(value = "当前页")
	private Integer currentPage; // 当前页

	@ApiModelProperty(value = "当前页数据列表")
	private List<T> items;// 当前页数据

	@ApiModelProperty(value = "当前页数据-<数据格式为 key-value 时使用>")
	private T item;

	public PageResult() {
	}

	public PageResult(Long total, List<T> items) {
		this.total = total;
		this.items = items;
	}

	public PageResult(Long total, T item) {
		this.total = total;
		this.item = item;
	}

	public PageResult(Long total, Integer totalPage, List<T> items) {
		this.total = total;
		this.totalPage = totalPage;
		this.items = items;
	}

	public PageResult(Long total, Integer totalPage, T item) {
		this.total = total;
		this.totalPage = totalPage;
		this.item = item;
	}

	public PageResult(Long total, Integer totalPage, Integer currentPage, List<T> items) {
		this.total = total;
		this.totalPage = totalPage;
		this.currentPage = currentPage;
		this.items = items;
	}

	public PageResult(Long total, Integer totalPage, Integer currentPage, T item) {
		this.total = total;
		this.totalPage = totalPage;
		this.currentPage = currentPage;
		this.item = item;
	}

	public PageResult(Page<T> pageInfo) {
		this.total = pageInfo.getTotal();
		this.totalPage = pageInfo.getPages();
		this.currentPage = pageInfo.getPageNum();
		this.items = pageInfo;
	}

}
